/*************************************************************************
Title........ streamsnd.h
Date......... Wed Jul 14 10:11:03  1999
Author....... Mike Ballinger
*************************************************************************/
#ifndef INTERFACE_AUDIOSTREAM
#define INTERFACE_AUDIOSTREAM

/****	Defines */
#define AUDIO_LEFT0                     0			/* Left channel 0*/
#define AUDIO_RIGHT0                    1			/* Right channel 0*/
#define AUDIO_STEREO0                   2			/* Left & Right channel 0*/
#define AUDIO_LEFT1                     3			/* Left channel 1*/
#define AUDIO_RIGHT1                    4			/* Right channel 1*/
#define AUDIO_STEREO1                   5			/* Left & Right channel 1*/
#define AUDIO_LOOPED                    16			/* Loop sample*/

#define CHANNEL_LEFT0					0			/* Left or stereo channel*/
#define CHANNEL_RIGHT0					1			/* Right or not active if stereo on LEFT*/
#define CHANNEL_LEFT1					2
#define CHANNEL_RIGHT1					3
#define NUM_CHANNELS					4

/**** ERROR CODES */
#define WAVERR_NOTFOUND					1			/* Requested WAV file not found*/
#define WAVERR_READ						2			/* Error reading WAV file*/
#define WAVERR_NORIFF					3			/* No "RIFF" header*/
#define WAVERR_NOWAVE					4			/* No "WAVE" header*/
#define WAVERR_NOFMT					5			/* No "fmt " header*/
#define WAVERR_NOTPCM					6			/* Not PCM encoded file*/
#define WAVERR_CHANNELS					7			/* Only MONO & STEREO supported*/
#define WAVERR_SAMPLERATE				8			/* Unsupported sample rate (only 16,000 & 32,000 allowed)*/
#define WAVERR_CHANNELINUSE				9			/* Audio channel currently in use*/
#define WAVERR_UNSUPPORTEDFMT			10			/* Unsupported format*/

#define	INITERR							1

#define	INITERR_FILENOTFOUND			1

#define	PLAYERR_CHANNELBUSY				1
#define	PLAYERR_INVALIDCHANNEL			2

#define	ERR_INVALIDCHANNEL				-1

/* SOUND FORMATS */
#define FORMAT_PCM						1			/* Uncompressed*/
#define FORMAT_ALAW						6			/* A-law compressed*/
#define FORMAT_ULAW						7			/* U-law compressed*/

#define SECTOR_BYTES					512
#define SECTOR_WORDS					(SECTOR_BYTES / 2)


#define MONO                            1
#define STEREO                          2

/* FILE OPENING */
#define PASS							0


/* Sound State Machine Definition */
typedef	enum
{
	SNDSTATE_CHANNEL_LEFT0,
	SNDSTATE_CHANNEL_RIGHT0,
	SNDSTATE_CHANNEL_LEFT1,
	SNDSTATE_CHANNEL_RIGHT1
} SNDSTATE;

typedef	enum
{
	CHANNEL_IDLE,
	CHANNEL_START,
	CHANNEL_PLAY,
	CHANNEL_STOPING,
	CHANNEL_WAVERROR,
	CHANNEL_INVALIDCHANNEL,
} CHANNELSTATUS;


/*
	Sampels required = buffer size / (1/16000)
	buffer size is equal number of states is sound state machine * 10ms + spare currently 5ms

	current number of states is 4 so (4 * 10ms) + 5ms / ((1/16000)) = 720

*/
#define SAMPLES_REQUIRED				720

#define CACHE_SECTORS					3
#define CACHE_SECTORS32K				6
#define CACHE_SECTORSCOMPRESSED			2
#define CACHE_SECTORSCOMPRESSED32K		4

#define MAXBUFFER						(SECTOR_WORDS * CACHE_SECTORS32K)

#define WAV_DMA_BUFFER_SIZE				( 4 * MAXBUFFER )	/* 5 file sectors*/

typedef struct wavfileinfo
{
	FILE_DESCRIPTOR file;					/* File pointer */
	UBYTE	channel;						/* Output channel, e.g. AUDIO_STEREO0*/
	UBYTE	looped;							/* 0 if not looped, 1 if looped.*/
	ULONG	fileLength;						/* Overall WAV file length (unused)*/
	ULONG	audioLength;					/* Audio data length (in WORDS)*/
	ULONG	audioPos;						/* Current position in sample data*/
	UWORD	numChannels;					/* Number of channels, e.g: 1 = mono, 2 = stereo etc*/
	ULONG	sampleRate;						/* Sample rate, e.g. 32000*/
	UWORD	bitsPerSample;					/* Bits per sample, e.g. 8, 16 etc*/
	ULONG	fileBufferPos;					/* Position in file buffer*/
	ULONG	audioBegin;						/* Beginning of audio data in file (offset in WORDS)*/
	UWORD	format;							/* FORMAT_PCM / FORMAT_ALAW / FORMAT_ULAW*/
	ULONG	BufferEnd;						/* End of file buffer*/
	CHANNELSTATUS	status;
	WORD	fileBuffer[MAXBUFFER];			/* From initial 1 sector read*/
} WAVFILEINFO;

typedef	struct wavdmavars
{
	UBYTE	activeDMAbuffer;
	UBYTE	nextBufferToFill;
	ULONG	bufferUsed[2];						/* Number of bytes in channel 0 buffer*/
	UBYTE	dmaRequired[2];						/* A DMA is required on specified buffer. Occurs in 10ms interrupt.*/
    UBYTE	bufferready[2];						/* Flags Set When Buffer Ready*/
	WORD	buffer[2][WAV_DMA_BUFFER_SIZE];
} WAVDMAVARS;


typedef union
{
	UBYTE	bb[WAV_DMA_BUFFER_SIZE*2];
	WORD	bw[WAV_DMA_BUFFER_SIZE];
} PROCESSBUFER;


/****	Prototypes */

#include	"streamsnd.p"

#endif
/*************************************************************************/
